#!/usr/bin/perl

#┌─────────────────────────────────
#│ ImageAlbum v4.02 : album.cgi - 2013/01/03
#│ Copyright (c) KentWeb
#│ http://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  Image Album v5.2 : album.cgi - 2022/10/01
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;
use File::ReadBackwards;

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# アクセス制限
passwd(%in) if ($cf{enter_pwd} ne '');

# 条件分岐
if ($in{mode} eq "detail") { detail(); }
if ($in{mode} eq "regist" && $cf{res_mode}) { regist(); }
if ($in{mode} eq "archv") { archv_log(); }
if ($in{mode} eq "compwd") { com_pwdform(); }


album_list();

#-----------------------------------------------------------
#  リスト一覧
#-----------------------------------------------------------
sub album_list {
	$in{sort} =~ s/\D//g;
	$in{cat} =~ s/\D//g;
	$in{pg} =~ s/\D//g;

	# カテゴリメニュー作成
	my ($list,$cat) = make_cate();

	# ページ数
	my $pg = $in{pg} || 0;

	# キーワードを配列化
	$in{word} =~ s/　/ /g;
	my @wd = split(/\s+/,$in{word});
	
	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';

		# データ読み出し
		my ($i,@log);
	if ($in{sort} == 1) {
		my $bw = File::ReadBackwards->new("$cf{datfile}");
		while( defined( my $log = $bw->readline ) ) {
			my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/, $log);
				next if ($in{cat} != $cat && $cat ne '');

			$i++;
			next if ($i < $pg + 1);
			next if ($i > $pg + $cf{pg_max});
			
        	push(@log,$log);
    		}
	} else {
		open(IN,"$cf{datfile}") or &error("open err: $cf{datfile}");
		while (<IN>) {
			chomp;
			my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/);

				# カテゴリ対応
				next if ($in{cat} != $cat && $cat ne '');

			if ($in{find} && $in{word} ne '') {
				my $flg;
				for my $wd (@wd) {
					if ("$sub $msg" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
						$flg++;
					} else {
						$flg = 0;
						last;
					}
				}
				next if (!$flg);
			}

			$i++;
			next if ($i < $pg + 1);
			next if ($i > $pg + $cf{pg_max});

			push(@log,$_);
		}
		close(IN);
	}

	# 繰越ページ
	my $pgbtn = make_pager($i,$pg);

	# サイドデータ
	my $archive = read_archive();
	my $topic = read_topic();

	# home or logoff
	my $home = $cf{enter_pwd} eq '' ? $cf{logoffurl} : "$cf{album_cgi}?mode=logoff";

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/album.html") or &error("open err: album.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置き換え
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s/!ttl_msg!/$cf{ttl_msg}/g;
	$tmpl =~ s|!album_css!|$cf{cmnurl}/album.css|g;
	$tmpl =~ s/!album_cgi!/$cf{album_cgi}/g;
	$tmpl =~ s/!admin!/$cf{admin_cgi}/g;
	$tmpl =~ s/!homepage!/$home/g;
	$tmpl =~ s/<!-- page_btn -->/$pgbtn/g;
	$tmpl =~ s/<!-- op_cate -->/op_cate()/e;
	$tmpl =~ s/<!-- op_sort -->/op_sort()/e;
	$tmpl =~ s/!word!/$in{word}/g;
	$tmpl =~ s/!cat!/$in{cat}/g;

	# カテゴリ一覧
	$tmpl =~ s/!cate_list!/$$list/;
	$tmpl =~ s/!cat-title!/$$cat{$in{cat}}/g;

	# 最新記事一覧
	$tmpl =~ s/<!-- topic -->/$topic/;
	$tmpl =~ s/!topic_count!/$cf{max_topics}/g;

	# アーカイブ一覧
	$tmpl =~ s/<!-- archive -->/$archive/;

	if (!$in{find}) {
		$tmpl =~ s|<!-- find -->.+?<!-- /find -->||s;
	}

	if ($cf{enter_pwd} eq '') {
		$tmpl =~ s|<!-- enter -->.+?<!-- /enter -->||s;
	}

	# 分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3) : &error("テンプレート不正");

	# 時間
	my $now = time;

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	my ($i,$tmp);
	foreach (@log) {
		chomp;
		$i++;
		my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/);
		$res ||= 0;

		# newマーク
		my $new_mark;
		if ($now - $cf{new_time}*60*60 < $time) {
			$new_mark = " $cf{new_mark}";
		}

			# 日時
			my $date;
			if ($time > 0) {
				my ($sec,$min,$hour,$day,$mon,$year) = (localtime($time))[0..5];
				$date = sprintf("%4d/%02d/%02d %02d:%02d:%02d",
							$year+1900,$mon+1,$day,$hour,$min,$sec);
			}

		# 三段組
		if ($i % 3 == 1) { $tmp = $loop; }

		($w,$h) = resize($w,$h,$cf{max_w},$cf{max_h});

		$tmp =~ s|!number-$i!|( 登録No. $no )|;
		$tmp =~ s/!date-$i!/$date/g;
		$tmp =~ s|!$i!|<a href="$cf{uplurl}/$no$ex" class="lightview" data-lightview-title="$sub" data-lightview-group="album-set" data-lightview-group-options="skin: 'mac'"><img src="$cf{uplurl}/$no$ex" width="$w" height="$h"></a>|g;
		$tmp =~ s|!sub-$i!|<a href="$cf{album_cgi}?mode=detail&no=$no&cat=$in{cat}&sort=$in{sort}&pg=$in{pg}">$sub</a> (<span style="color:#288c66">$res</span>)|g;
		$tmp =~ s/!new_mark-$i!/$new_mark/g;

		if ($i % 3 == 0) {
			print $tmp;
			$i = 0;
		}
	}
	# 端数あり
	if ($i % 3) {
		$tmp =~ s|<div class="gallery">![1-3]!|&nbsp;|g;
		$tmp =~ s|!sub-[1-3]!</p>|&nbsp;|g;
		$tmp =~ s|!date-[1-3]!</p>|&nbsp;|g;
		$tmp =~ s|!number-[1-3]!</div>|&nbsp;|g;
		$tmp =~ s|<div id="new_mark">!new_mark-[1-3]!</div>|&nbsp;|g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  セレクト：並び順
#-----------------------------------------------------------
sub op_sort {
	$in{cat} =~ s/\D//g;
	my @sort = qw(昇順 降順);
	my $ret;
	for (0,1) {
		if ($in{sort} == $_) {
			$ret .= qq|<option value="$cf{index_cgi}?sort=$_&cat=$in{cat}" selected>$sort[$_]\n|;
		} else {
			$ret .= qq|<option value="$cf{index_cgi}?sort=$_&cat=$in{cat}">$sort[$_]\n|;
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  セレクト：カテゴリ
#-----------------------------------------------------------
sub op_cate {
	#my $ret = qq|<option value="$cf{album_cgi}">[全表示]\n|;
	my $ret;
	open(IN,"$cf{catfile}") or error("open err: $cf{catfile}");
	while (<IN>) {
		my ($no,$cat) = split(/<>/);
		
		if ($in{cat} == $_) {
			$ret .= qq|<option value="$cf{album_cgi}?cat=$no" selected>$cat\n|;
		} else {
			$ret .= qq|<option value="$cf{album_cgi}?cat=$no">$cat\n|;
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  最新記事INDEX作成
#-----------------------------------------------------------
sub read_topic {

	# 時間
	my $now = time;

	# 最新記事リスト表示
	my ($i,$topic,@top_ttl);
	open(IN,"$cf{datfile}") or error("open err: $cf{datfile}");
	while (<IN>) {
		my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/);

		if (defined($no)) { push(@top_ttl,$_); }

		# newマーク
		my $new;
		if ($now - $cf{new_time}*60*60 < $time) {
			$new = " $cf{new_mark2}";
		}

		$i++;
		last if ($i >= $cf{max_topics} + 1);

		$topic .= qq|<div id="topi-ttl">○ <a href="$cf{album_cgi}?mode=detail&no=$no&cat=$in{cat}&sort=$in{sort}&pg=$in{pg}">$sub</a> (<span style="color:#288c66">$res</span>) $new</div>|;

	}
	close(IN);

	foreach my $ttl (@top_ttl) {
		my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/, $ttl);

			$topic = qq|$topic\n|;
	}
	# 結果を返す
	return $topic;
}

#-----------------------------------------------------------
#  月間アーカイブ記事
#-----------------------------------------------------------
sub archv_log {

	# カテゴリメニュー作成
	my ($list,$cat) = make_cate();

	# 記事抽出条件
	$in{cond} =~ s/\D//g;
	$in{mon} =~ s/\t//g;

	# 記事抽出
	my ($hit,@arch) = archv_pickup($in{mon},$in{cond}) if ($in{mon} ne '');

	# ペイジャー
	my $archive_pager = $hit ? archive_pager($hit,$in{pg},"&mode=archv") : '';

	# サイドデータ
	my $archive = read_archive();
	my $topic = read_topic();

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/archv.html") or &error("open err: archv.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置き換え
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s/!ttl_msg!/$cf{ttl_msg}/g;
	$tmpl =~ s|!album_css!|$cf{cmnurl}/album.css|g;
	$tmpl =~ s/!album_cgi!/$cf{album_cgi}/g;
	$tmpl =~ s/!homepage!/$cf{homepage}/g;
	$tmpl =~ s/<!-- op_cate -->/op_cate()/e;

	if ($cf{enter_pwd} eq '') {
		$tmpl =~ s|<!-- enter -->.+?<!-- /enter -->||s;
	}

	# 日付フォーマット
	my ($y,$m) = $in{mon} =~ /^(\d{4})\/(\d{2})$/ && ($1,$2);
	my $date = "$y年$m月";
	$tmpl =~ s/!ym!/$date/g;

	# カテゴリ一覧
	$tmpl =~ s/!cate_list!/$$list/;
	$tmpl =~ s/!cat-title!/$$cat{$in{cat}}/g;

	# 最新記事一覧
	$tmpl =~ s/<!-- topic -->/$topic/;
	$tmpl =~ s/!topic_count!/$cf{max_topics}/g;

	# アーカイブ一覧
	$tmpl =~ s/<!-- archive -->/$archive/;

	if ($in{mon}) {
		$tmpl =~ s/!result!/$hit/g;
		$tmpl =~ s/<!-- pager -->/$archive_pager/g;
	} else {
		$tmpl =~ s/<!-- archive_begin -->.+<!-- archive_end -->//s;
	}

	# 分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3) : &error("テンプレート不正");

	# 時間
	my $now = time;

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	my ($i,$tmp);
	foreach (@arch) {
		chomp;
		$i++;
		my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/);
		$res ||= 0;

		# newマーク
		my $new_mark;
		if ($now - $cf{new_time}*60*60 < $time) {
			$new_mark = " $cf{new_mark}";
		}

		# 三段組
		if ($i % 3 == 1) { $tmp = $loop; }

		($w,$h) = resize($w,$h,$cf{max_w},$cf{max_h});
		$tmp =~ s|!$i!|<a href="$cf{uplurl}/$no$ex" class="lightview" data-lightview-title="$sub" data-lightview-group="album-set" data-lightview-group-options="skin: 'mac'"><img src="$cf{uplurl}/$no$ex" width="$w" height="$h"></a>|g;
		$tmp =~ s|!sub-$i!|<a href="$cf{album_cgi}?mode=detail&no=$no">$sub</a> (<span style="color:#288c66">$res</span>)|g;
		$tmp =~ s/!new_mark-$i!/$new_mark/g;

		if ($i % 3 == 0) {
			print $tmp;
			$i = 0;
		}
	}
	# 端数あり
	if ($i % 3) {
		$tmp =~ s|<div class="gallery">![1-3]!|&nbsp;|g;
		$tmp =~ s|!sub-[1-3]!</div>|&nbsp;|g;
		$tmp =~ s|<div id="new_mark">!new_mark-[1-3]!</div>|&nbsp;|g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  月間アーカイブ記事抽出
#-----------------------------------------------------------
sub archv_pickup {
	my ($word,$cond) = @_;

	$in{pg} =~ s/\D//g;
	$in{pg} ||= 0;

	# キーワードを配列化
	$word =~ s/　/ /g;
	my @wd = split(/\s+/,$word);
	
	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';

	# 記事抽出処理
	my ($i,@arch);
	open(IN,"$cf{datfile}") or error("open err: $cf{datfile}");
	while (<IN>) {
		my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/);

	# 日時
	my $date;
	if ($time > 0) {
		my ($day,$mon,$year) = (localtime($time))[3..5];
		$date = sprintf("%4d/%02d/%02d",$year+1900,$mon+1,$day);
	}

		my $flg;
		foreach my $wd (@wd) {
			if ("$date" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
				$flg++;
				if ($cond == 0) { last; }
			} else {
				if ($cond == 1) { $flg = 0; last; }
			}
		}
		next if (!$flg);

			$i++;
			if ($cf{archive_data} > 0) {
				next if ($i < $in{pg} + 1);
				next if ($i > $in{pg} + $cf{archive_data});
			}

		push(@arch,$_);
	}
	close(IN);

	# 検索結果
	return ($i,@arch);
}

#-----------------------------------------------------------
#  アーカイブ画面
#-----------------------------------------------------------
sub read_archive {

	# アーカイブ生成
	my (%mons,@top_ttl,$archive);
	open(IN,"$cf{datfile}") or error("open err: $cf{datfile}");
	while(<IN>) {
		my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/);

	# 日時
	my $date;
	if ($time > 0) {
		my ($day,$mon,$year) = (localtime($time))[3..5];
		$date = sprintf("%4d/%02d/%02d",$year+1900,$mon+1,$day);
	}
		# 年月
		my ($y,$m,$d) = $date =~ /^(\d+)\/(\d+)\/(\d+)/ && ($1,$2,$3);
		$mons{"$y$m"}++;

		$archive = put_archive(%mons);

	}
	close(IN);

	$archive = qq|$archive\n|;

	# 結果を返す
	return $archive;
}

#-----------------------------------------------------------
#  月間アーカイブ作成
#-----------------------------------------------------------
sub put_archive {
	my %mon = @_;

	my ($ret,$i);
	for ( sort{ $b <=> $a }keys %mon ) {
		/^(\d{4})(\d{2})/;

		$i++;
		last if ($i >= $cf{max_archive} + 1);

		$ret .= qq|<div id="topi-ttl">○ <a href="$cf{bbs_cgi}?mode=archv&mon=$1/$2">$1年$2月</a> (<span style="color:#941f57">$mon{$_}</span>)</div>\n|;
	}
	# 結果を返す
	return $ret;
}

#-----------------------------------------------------------
#  月間アーカイブページ送り作成
#-----------------------------------------------------------
sub archive_pager {
	my ($i,$pg,$stat) = @_;
	
	# ページ繰越数定義
	my $next = $pg + $cf{archive_data};
	my $back = $pg - $cf{archive_data};

	# パラメータ
	my $param = "&mon=$in{mon}";

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><a href="#" class="active"><span>$x</span></a></li>\n!);
			} else {
				push(@pg,qq!<li><a href="$cf{album_cgi}?$stat&pg=$y$param"><span>$x</span></a></li>\n!);
			}
			$x++;
			$y += $cf{archive_data};
			$z -= $cf{archive_data};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li class="pre"><a href="$cf{album_cgi}?$stat&pg=$back$param"><span>&laquo;</span></a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li class="next"><a href="$cf{album_cgi}?$stat&pg=$next$param"><span>&raquo;</span></a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<div class="pager">\n<ul class="pagination">\n$ret</ul></div>| : '';
}

#-----------------------------------------------------------
#  カテゴリメニュー作成
#-----------------------------------------------------------
sub make_cate {
	# カテゴリ引数
	$in{cat} =~ s/\D//g;

	# テンプレート読込
	open(IN,"$cf{tmpldir}/cate.html") or error("open err: cate.html");
	my $loop = join('', <IN>);
	close(IN);

	# カテゴリファイル
	my ($i,$list,%cat);
	open(IN,"$cf{catfile}") or error("open err: $cf{catfile}");
	while(<IN>) {
		$i++;
		my ($no,$cat) = split(/<>/);

		# 引数なしのとき
		if (!$in{cat} && $i == 1) { $in{cat} = $no; }

		# ハッシュ化
		$cat{$no} = $cat;

		# リスト作成
		my $tmp = $loop;
		$tmp =~ s|!cat_name!|<a href="$cf{album_cgi}?cat=$no">$cat</a>|;
		$list .= $tmp;
	}
	close(IN);

	# 結果を返す
	return (\$list,\%cat);
}

#-----------------------------------------------------------
#  詳細画面
#-----------------------------------------------------------
sub detail {

	# 汚染チェック
	my $resfile;
	if ($in{no} =~ /^(\d+)$/) {
		$resfile = "$cf{datadir}/$1.cgi";
	} else {
		error("不正なアクセスです");
	}

	# データ読み出し
	my (@log,%cat);
	open(IN,"$cf{datfile}") or &error("open err: $cf{datfile}");
	while (<IN>) {
	my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/);

		if ($in{no} == $no) { 

			push(@log,$_);
		}
	}
	close(IN);

	# コメント返信ログ
	my @reply;
	open(IN,"$resfile");
	while(<IN>) {
		my ($num,$date,$name,$msg,$url,$host,$time,$cat,$ex,$w,$h,$pw) = split(/<>/);

			push(@reply,$_);
	}
	close(IN);

	# クッキー取得
	my ($ck_nam,$ck_url) = &get_cookie;
	$ck_url ||= 'http://';

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/read.html") or &error("open err: read.html");
	my $tmpl = join('', <IN>);
	close(IN);

	open(IN,"$cf{tmpldir}/res.html") or error("open err: res.html");
	my $resloop = join('', <IN>);
	close(IN);

	# 返信機能なしのとき
	if ($cf{res_mode} == 0) {
		$tmpl =~ s/<!-- form_begin -->.+<!-- form_end -->//s;
	}

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl,$resloop);

	# 画像認証作成
	my ($str_plain,$str_crypt);
	if ($cf{use_captcha} > 0 && $cf{res_mode}) {
		require $cf{captcha_pl};
		($str_plain, $str_crypt) = cap::make( $cf{captcha_key}, $cf{cap_len} );
		$foot =~ s/!str_crypt!/$str_crypt/g;
	} else {
		$foot =~ s/<!-- captcha_begin -->.+<!-- captcha_end -->//s;
	}

	# 文字置換
	for ($head,$foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!cgi_title!/$cf{cgi_title}/g;
		s/!ttl_msg!/$cf{ttl_msg}/g;
		s|!album_css!|$cf{cmnurl}/album.css|g;
		s/!fm-nam!/$ck_nam/g;
		s/!fm-url!/$ck_url/g;
		s/!no!/$in{no}/g;
		s/!cat!/$in{cat}/g;
		s/!sort!/$in{sort}/g;
		s/!pg!/$in{pg}/g;
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach my $log (@log) {
		my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/, $log);
			$msg = autolink($msg) if ($cf{autolink});
			$msg =~ s/&amp;/&/g;

			# newマーク
			my $new_mark;
			if (time - $cf{new_time}*60*60 < $time) {
				$new_mark = " $cf{new_mark}";
			}

			# 日時
			my $date;
			if ($time > 0) {
				my ($sec,$min,$hour,$day,$mon,$year) = (localtime($time))[0..5];
				$date = sprintf("%4d/%02d/%02d %02d:%02d:%02d",
							$year+1900,$mon+1,$day,$hour,$min,$sec);
			}

			# 画像サイズ調整
			($w,$h) = resize($w,$h,$cf{max_w2},$cf{max_h2});

		# 記事書き出し
		my $tmp = $loop;
		$tmp =~ s|!image!|<a href="$cf{uplurl}/$no$ex" class="lightview" data-lightview-title="$sub" data-lightview-group="album-set" data-lightview-group-options="skin: 'mac'"><img src="$cf{uplurl}/$no$ex" width="$w" height="$h"></a>|g;
		$tmp =~ s/!subject!/$sub/g;
		$tmp =~ s/!new_mark!/$new_mark/g;
		$tmp =~ s/!comment!/$msg/g;
		$tmp =~ s/<!-- date -->/$date/g;
		print $tmp;

	# レス記事展開
	foreach (@reply) {
		my ($num,$date,$name,$msg,$url,$host,$time,$cat,$ex,$w,$h,$pw) = split(/<>/);
		$name = qq|<a href="$url" target="_blank">$name</a>| if ($url);
		$msg = autolink($msg) if ($cf{autolink});

		my $clip;
		# 添付あり
		if ($ex) {

			# 添付画像
			if ($ex =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{resupldir}/$in{no}-$num-s$ex") {
					$op = qq|src="$cf{resuplurl}/$in{no}-$num-s$ex"|;
				} else {
					($w,$h) = resize_res($w,$h);
					$op = qq|src="$cf{resuplurl}/$in{no}-$num$ex" width="$w" height="$h"|;
				}
				$clip .= qq|<div id="clip"><a href="$cf{resuplurl}/$in{no}-$num$ex" class="lightview" data-lightview-title="$name" data-lightview-group="album-set" data-lightview-group-options="skin: 'mac'"><img $op border="0" class="snapshot"></a></div>|;
			}
		}

		my $tmp = $resloop;
		$tmp =~ s/!no!/$in{no}/g;
		$tmp =~ s/!num!/$num/g;
		$tmp =~ s/!message!/$msg/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/<!-- ext -->/$clip/g;
		$tmp =~ s|!com_edit!|<a href="$cf{album_cgi}?mode=compwd&amp;no=$in{no}&amp;num=$num"><img src="$cf{cmnurl}/Edit.png" alt="" class="icon" /></a>|g;
		print $tmp;
	}
}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  コメント返信パスワードフォーム
#-----------------------------------------------------------
sub com_pwdform {

	$in{num} =~ s/\D//g;

	# クッキー取得
	my @cook = get_cookie();
	$cook[2] ||= 'http://';

	# 記事
	my ($name,$pwd);
	open(DAT,"+< $cf{datadir}/$in{no}.cgi");
	eval "flock(DAT, 2);";
	while (<DAT>) {
		my ($num,$date,$name,$msg,$url,$host,$time,$cat,$ex,$w,$h,$pw) = split(/<>/);
			if ($in{num} == $num) {
				$name = $name;
				$pwd = $pw;
				last;
			}
	}
	close(DAT);

	# パスワード未設定
	error("このコメントはパスワード未設定のため編集できません") if ($pwd eq '');

	my ($num,$date,$name,$msg,$url,$host,$time,$cat,$ex,$w,$h,$pw) = split(/<>/);
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a>|;
		if (length($msg) > 110) { $msg = substr($msg,0,120); $msg .= '.....(略)'; }

		my $clip;
		# 添付あり
		if ($ex) {

			# 添付画像
			if ($ex =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{resupldir}/$in{no}-$num-s$ex") {
					$op = qq|src="$cf{resuplurl}/$in{no}-$num-s$ex"|;
				} else {
					($w,$h) = resize_res($w,$h);
					$op = qq|src="$cf{resuplurl}/$in{no}-$num$ex" width="$w" height="$h"|;
				}
				$clip .= qq|<div id="clip"><a href="$cf{resuplurl}/$in{no}-$num$ex" class="lightview" data-lightview-title="$name" data-lightview-group="album-set" data-lightview-group-options="skin: 'mac'"><img $op border="0" class="snapshot"></a></div>|;
			}
		}

	open(IN,"$cf{tmpldir}/res_pwdform.html") or &error("open err: res_pwdform.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!name!/$name/g;
	$tmpl =~ s/!num!/$num/g;
	$tmpl =~ s/!no!/$in{no}/g;
	$tmpl =~ s/!message!/$msg/g;
	$tmpl =~ s/!date!/$date/g;
	$tmpl =~ s/!url!/$url/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s/!ttl_msg!/$cf{ttl_msg}/g;
	$tmpl =~ s|!album_css!|$cf{cmnurl}/album.css|g;
	$tmpl =~ s/!pwd!/$pwd/g;
	$tmpl =~ s/<!-- ext -->/$clip/g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  返信書き込み
#-----------------------------------------------------------
sub regist {
	# セキュリティチェック
	if ($cf{postonly} && $ENV{REQUEST_METHOD} ne "POST") {
		error("不正なリクエストです");
	}

	# 汚染チェック
	my $resfile;
	if ($in{no} =~ /^(\d+)$/) {
		$resfile = "$cf{datadir}/$1.cgi";
	} else {
		error("不正なアクセスです");
	}

	# 画像認証チェック
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		if ($in{captcha} !~ /^\d{$cf{cap_len}}$/) {
			error("画像認証が入力不備です。<br>投稿フォームに戻って再読込み後、再入力してください");
		}

		# 投稿キーチェック
		# -1 : キー不一致
		#  0 : 制限時間オーバー
		#  1 : キー一致
		my $chk = cap::check($in{captcha},$in{str_crypt},$cf{captcha_key},$cf{cap_time},$cf{cap_len});
		if ($chk == 0) {
			error("画像認証が制限時間を超過しました。<br>投稿フォームに戻って再読込み後、指定の数字を再入力してください");
		} elsif ($chk == -1) {
			error("画像認証が不正です。<br>投稿フォームに戻って再読込み後、再入力してください");
		}
	}

	# 入力チェック
	my $err;
	if ($in{url} eq "http://") { $in{url} = ''; }
	if ($in{url} ne '' && $in{url} !~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/) {
		$err .= "参照先URLの入力内容が不正です<br>";
	}
	if ($in{name} eq "") { $err .= "名前が入力されていません<br>"; }
	if ($in{comment} eq "") { $err .= "コメントが入力されていません<br>"; }
	if ($err) { error($err); }

	# 不要改行カット
	$in{name} =~ s/<br>//g;
	$in{comment} =~ s/(<br>)+$//g;

	# ホストチェック
	my $host = axscheck();

	# 時間取得
	my $time = time;
	my ($min,$hour,$mday,$mon,$year) = (localtime($time))[1..5];
	my $date = sprintf("%04d/%02d/%02d-%02d:%02d",
					$year+1900,$mon+1,$mday,$hour,$min);

	# 削除キー暗号化
	my $crypt = encrypt($in{pwd}) if ($in{pwd} ne "");

	# 記事数
	my $new_res;

	# 既存データあり
	if (-f $resfile) {

		open(DAT,"+< $resfile") or &error("open err: $resfile");
		eval "flock(DAT, 2);";
		my @data = <DAT>;

		# 記事No
		my ($no,$hos,$tim,$cat) = (split(/<>/, $data[0]))[0,5,6,7];
		$no++;

		# 画像アップ
		my ($ex,$w,$h) = upload($no);

		# 連続投稿チェック
		my $flg;
		if ($cf{regCtl} == 1) {
			if ($host eq $hos && $time - $tim < $cf{wait}) { $flg = 1; }
		} elsif ($cf{regCtl} == 2) {
			if ($time - $tim < $cf{wait}) { $flg = 1; }
		}
		if ($flg) {
			error("現在投稿制限中です。もうしばらくたってから投稿をお願いします");
		}

		$new_res = @data + 1;

		# 記事数調整
		while ($cf{res_max} <= @data) { pop(@data); }
		unshift(@data,"$no<>$date<>$in{name}<>$in{comment}<>$in{url}<>$host<>$time<>$cat<>$ex<>$w<>$h<>$crypt<>\n");

		# 更新
		seek(DAT, 0, 0);
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);

	# 新規データ生成
	} else {
		$new_res = 1;

		open(DAT,"+> $resfile") or &error("write err: $resfile");
		eval "flock(DAT, 2);";
		my @data = <DAT>;

		my ($no,$hos,$tim,$cat) = (split(/<>/, $data[0]))[0,5,6,7];
		$no++;

		# 画像アップ
		my ($ex,$w,$h) = upload($no);

		print DAT "1<>$date<>$in{name}<>$in{comment}<>$in{url}<>$host<>$time<>$in{cat}<>$ex<>$w<>$h<>$crypt<>\n";
		close(DAT);

		chmod(0666, $resfile);
	}

	# indexデータ
	my ($msub,@log);
	open(DAT,"+< $cf{datfile}") or &error("open err: $cf{datfile}");
	eval "flock(DAT, 2);";
	while (<DAT>) {
		chomp;
		my ($no,$sub,$msg,$ex,$w,$h,$res,$time,$cat) = split(/<>/);

		if ($in{no} == $no) {
			$msub = $sub;
			$_ = "$no<>$sub<>$msg<>$ex<>$w<>$h<>$new_res<>$time<>$cat<>";
		}
		push(@log,"$_\n");
	}
	seek(DAT, 0, 0);
	print DAT @log;
	truncate(DAT, tell(DAT));
	close(DAT);

	# クッキー
	if ($in{cook} == 1) {
		# 保存
		set_cookie($in{name},$in{url});
	} else {
		# 消去
		print "Set-Cookie: ImageAlbum=; expires=Mon, 01-Apr-1998 00:00:00 GMT\n";
	}

	# メール通知
	sendmail($msub,$host,$date) if ($cf{mailing} == 1);

	# 完了
	message("記事を受理しました");
}

#-----------------------------------------------------------
#  crypt暗号
#-----------------------------------------------------------
sub encrypt {
	my $in = shift;

	my @wd = ('a'..'z', 'A'..'Z', '0'..'9', '.', '/');
	srand;
	my $salt = $wd[int(rand(@wd))] . $wd[int(rand(@wd))];
	crypt($in, $salt) || crypt ($in, '$1$' . $salt);
}

#-----------------------------------------------------------
#  画像アップロード
#-----------------------------------------------------------
sub upload {
	my $num = shift;

	my $resnum;
	if ($in{no} =~ /^(\d+)$/) {
		$resnum = "$1";
	}

	my $fname = $cgi->param_filename('upfile');
	if ($fname =~ /(\.jpe?g|\.png|\.gif|\.bmp)$/i) {
		my $ex = $1;
		if ($ex eq '.jpeg') { $ex = '.jpg'; }

		# アップファイル定義
		my $imgfile = "$cf{resupldir}/$resnum-$num$ex";

		# 書き込み
		open(UP,"+> $imgfile") or error("up err: $imgfile");
		binmode(UP);
		print UP $in{upfile};
		close(UP);

		# 画像サイズ取得
		my ($w,$h);
		if ($ex eq ".jpg") { ($w,$h) = &j_size($imgfile); }
		elsif ($ex eq ".gif") { ($w,$h) = &g_size($imgfile); }
		elsif ($ex eq ".png") { ($w,$h) = &p_size($imgfile); }
		elsif ($ex eq ".bmp") { ($w,$h) = &b_size($imgfile); }

		chmod(0666,$imgfile);

		return ($ex,$w,$h);
	}
	return;
}

#-----------------------------------------------------------
#  JPEGサイズ認識
#-----------------------------------------------------------
sub j_size {
	my $jpg = shift;

	my ($h, $w, $t);
	open(IMG,"$jpg");
	binmode(IMG);
	read(IMG, $t, 2);
	while (1) {
		read(IMG, $t, 4);
		my ($m, $c, $l) = unpack("a a n", $t);

		if ($m ne "\xFF") {
			$w = $h = 0;
			last;
		} elsif ((ord($c) >= 0xC0) && (ord($c) <= 0xC3)) {
			read(IMG, $t, 5);
			($h, $w) = unpack("xnn", $t);
			last;
		} else {
			read(IMG, $t, ($l - 2));
		}
	}
	close(IMG);

	return ($w, $h);
}

#-----------------------------------------------------------
#  GIFサイズ認識
#-----------------------------------------------------------
sub g_size {
	my $gif = shift;

	my $data;
	open(IMG,"$gif");
	binmode(IMG);
	sysread(IMG, $data, 10);
	close(IMG);

	if ($data =~ /^GIF/) { $data = substr($data, -4); }
	my $w = unpack("v", substr($data, 0, 2));
	my $h = unpack("v", substr($data, 2, 2));

	return ($w, $h);
}

#-----------------------------------------------------------
#  PNGサイズ認識
#-----------------------------------------------------------
sub p_size {
	my $png = shift;

	my $data;
	open(IMG,"$png");
	binmode(IMG);
	read(IMG, $data, 24);
	close(IMG);

	my $w = unpack("N", substr($data, 16, 20));
	my $h = unpack("N", substr($data, 20, 24));

	return ($w, $h);
}

#-----------------------------------------------------------
#  BMPサイズ
#-----------------------------------------------------------
sub b_size {
	my $bmp = shift;

	my $buf;
	open(BMP, "$bmp") || return (0,0);
	binmode(BMP);
	seek(BMP, 0, 0);
	read(BMP, $buf, 6);
	seek(BMP, 12, 1);
	read(BMP, $buf, 8);
	my ($w, $h) = unpack("VV", $buf);

	return ($w, $h);
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;

	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<p style="margin-top:2em;text-align:center;font-family:Verdana,Helvetica,Arial;font-size:11pt;">
 [ <a href="https://www.kent-web.com/" target="_top">ImageAlbum : オリジナル公開終了</a> ]<br />
- <a href="https://www.transformsite.com/" target="_top">$cf{version} - Edited by TRANSFORM</a> -
</p>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  サイズ縮小
#-----------------------------------------------------------
sub resize {
	my ($w,$h,$max_w,$max_h) = @_;

	# 画像表示縮小
	if ($w > $max_w || $h > $max_h) {
		my $w2 = $max_w / $w;
		my $h2 = $max_h / $h;
		my $key;
		if ($w2 < $h2) { $key = $w2; } else { $key = $h2; }
		$w = int ($w * $key) || 1;
		$h = int ($h * $key) || 1;
	}
	return ($w,$h);
}

#-----------------------------------------------------------
#  アクセス制限
#-----------------------------------------------------------
sub axscheck {
	# IP&ホスト取得
	my $host = $ENV{'REMOTE_HOST'};
	my $addr = $ENV{'REMOTE_ADDR'};

	if ($cf{gethostbyaddr} && ($host eq "" || $host eq $addr)) {
		$host = gethostbyaddr(pack("C4", split(/\./, $addr)), 2);
	}
	if ($host eq "") { $host = $addr; }

	# アクセスチェック
	my $flg;
	foreach ( split(/\s+/, $cf{denyhost}) ) {
		if (index($host,$_) >= 0) { $flg = 1; last; }
		if (index($addr,$_) >= 0) { $flg = 1; last; }
	}
	if ($flg) { error("アクセスを許可されていません"); }

	return $host;
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub set_cookie {
	my @cook = @_;

	# 国際標準時を定義
	my $gmt = $cgi->date_rfc1123(time + 90*24*60*60);

	# URLエンコード
	my $cookie;
	foreach (@cook) {
		s/(\W)/sprintf("%%%02X", unpack("C", $1))/eg;
		$cookie .= "$_<>";
	}

	# 格納
	print "Set-Cookie: ImageAlbum=$cookie; expires=$gmt\n";
}

#-----------------------------------------------------------
#  クッキー取得
#-----------------------------------------------------------
sub get_cookie {
	# クッキーを取得
	my $cookie = $ENV{HTTP_COOKIE};

	# 該当IDを取り出す
	my %cook;
	foreach ( split(/;/, $cookie) ) {
		my ($key, $val) = split(/=/);
		$key =~ s/\s//g;
		$cook{$key} = $val;
	}

	# データをURLデコードして復元
	my @cook;
	foreach ( split(/<>/, $cook{'ImageAlbum'}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2", $1)/eg;
		s/[&"'<>]//g;

		push(@cook,$_);
	}
	return @cook;
}

#-----------------------------------------------------------
#  完了メッセージ
#-----------------------------------------------------------
sub message {
	my ($msg) = @_;

	open(IN,"$cf{tmpldir}/message.html") or &error("open err: message.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!album_cgi!/$cf{album_cgi}/g;
	$tmpl =~ s/!message!/$msg/g;
	$tmpl =~ s|!album_css!|$cf{cmnurl}/album.css|g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  繰越ボタン
#-----------------------------------------------------------
sub make_pager {
	my ($i,$pg) = @_;

	# 繰越ページ定義
	$cf{pg_max} ||= 10;
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};

	# ページ繰越ボタン作成
	my (@pg,$param);
	if ($in{sort}) { $param .= "&amp;sort=$in{sort}"; }
	if ($in{cat} ne '') { $param .= "&amp;cat=$in{cat}"; }
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><a href="#" class="active"><span>$x</span></a></li>\n!);
			} else {
				push(@pg,qq!<li><a href="$cf{album_cgi}?pg=$y$param"><span>$x</span></a></li>\n!);
			}
			$x++;
			$y += $cf{pg_max};
			$z -= $cf{pg_max};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li class="pre"><a href="$cf{album_cgi}?pg=$back$param"><span>&laquo;</span></a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li class="next"><a href="$cf{album_cgi}?pg=$next$param"><span>&raquo;</span></a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<div class="pager">\n<ul class="pagination">\n$ret</ul></div>| : '';
}

#-----------------------------------------------------------
#  自動リンク
#-----------------------------------------------------------
sub autolink {
	local($_) = shift;

	s|(s?https?://[\w-.!~*'();/?:\@&=+\$,%#]+)|<a href="$1" target="_blank">$1</a>|g;
	$_;
}

#-----------------------------------------------------------
#  エラー画面
#-----------------------------------------------------------
sub error {
	my $err = shift;

	open(IN,"$cf{tmpldir}/error.html") or die;
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!error!/$err/g;
	$tmpl =~ s/!album_cgi!/$cf{album_cgi}/g;
	$tmpl =~ s/!cgi_title!/$cf{cgi_title}/g;
	$tmpl =~ s|!album_css!|$cf{cmnurl}/album.css|g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  メール送信
#-----------------------------------------------------------
sub sendmail {
	my ($sub,$host,$date) = @_;

	# 件名をMIMEエンコード
	require "lib/jacode.pl";
	my $msub = mime_unstructured_header("$subへのコメント");

	# コメント内の改行復元
	my $com = $in{comment};
	$com =~ s/<br>/\n/g;
	$com =~ s/&lt;/>/g;
	$com =~ s/&gt;/</g;
	$com =~ s/&quot;/"/g;
	$com =~ s/&amp;/&/g;
	$com =~ s/&#39;/'/g;

	# メール本文を定義
	my $body = <<"EOM";
$cf{cgi_title}にコメント投稿がありました。

投稿日：$date
ホスト：$host

件名  ：$sub
お名前：$in{name}
URL   ：$in{url}

$com
EOM

	# JISコード変換
	my $mbody;
	for my $tmp ( split(/\n/,$body) ) {
		jcode::convert(\$tmp,'jis','utf8');
		$mbody .= "$tmp\n";
	}
	
	# FROM行の定義
	my $from = $in{email} || $cf{mailto};
	
	# sendmailコマンド
	my $scmd = "$cf{sendmail} -t -i";
	if ($cf{sendm_f} == 1) { $scmd .= " -f $from"; }

	# 送信
	open(MAIL,"| $scmd") or error("送信失敗");
	print MAIL "To: $cf{mailto}\n";
	print MAIL "From: $from\n";
	print MAIL "Subject: $msub\n";
	print MAIL "MIME-Version: 1.0\n";
	print MAIL "Content-type: text/plain; charset=ISO-2022-JP\n";
	print MAIL "Content-Transfer-Encoding: 7bit\n";
	print MAIL "X-Mailer: $cf{version}\n\n";
	print MAIL "$mbody\n";
	close(MAIL);
}

